<?php

namespace Pinpoint\BlankProduct\Observer\Order;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;

/**
 * Class Cancel
 * @package Pinpoint\BlankProduct\Observer\Order
 */
class Cancel implements ObserverInterface
{
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;

    /**
     * @var \Magento\CatalogInventory\Model\Stock\Item
     */
    protected $stockItem;

    /**
     * Cancel constructor.
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\CatalogInventory\Model\Stock\Item $stockItem
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\CatalogInventory\Model\Stock\Item $stockItem
    ) {
        $this->productFactory = $productFactory;
        $this->stockItem = $stockItem;
    }

    /**
     * Execute Observer
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /** @var \Magento\Sales\Model\Order $order */
        $order = $observer->getData('order');

        foreach ($order->getItems() as $orderItem) {
            if ($product = $this->productFactory->create()->load($orderItem->getProductId())) {
                if ($product->getCustomAttribute('blank_products') &&
                    trim($product->getCustomAttribute('blank_products')->getValue()) != '') {

                    $blankProducts = explode(',', $product->getCustomAttribute('blank_products')->getValue());
                    $cancelledQuantity = $orderItem->getQtyCanceled();

                    $productStockItem = $this->stockItem->load($product->getId(), 'product_id');
                    $productStockItem->setQty($productStockItem->getQty() - $cancelledQuantity);
                    $productStockItem->save();

                    foreach ($blankProducts as $blankProduct) {
                        $blankProductData = explode("|", $blankProduct);

                        $_product = $this->productFactory->create()->loadByAttribute('sku', $blankProductData[0]);
                        $stockItem = $this->stockItem->load($_product->getId(), 'product_id');

                        $stockItem->setQty($stockItem->getQty() + ($cancelledQuantity * $blankProductData[1]));
                        $stockItem->save();
                    }
                }
            }
        }
    }
}