<?php

namespace Pinpoint\BlankProduct\Plugin\Model;

/**
 * Class ProductPlugin
 * @package Pinpoint\BlankProduct\Plugin\Model
 */
class ProductPlugin
{
    /** @var \Magento\Catalog\Model\ProductFactory ProductFactory */
    protected $_productFactory;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $_coreRegistry;

    /**
     * @var \Magento\Backend\Model\Auth\Session
     */
    protected $_authSession;

    /**
     * @var \Pinpoint\BlankProduct\Helper\Quantity
     */
    protected $_blankProductHelper;

    /**
     * @var string
     */
    const STOCKITEM_SALEABLE_REGISTRY = 'stockitem_getissaleable';

    /**
     * ProductPlugin constructor.
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \Magento\Framework\Registry $coreRegistry
     * @param \Magento\Backend\Model\Auth\Session $authSession
     * @param \Pinpoint\BlankProduct\Helper\Quantity $blankProductHelper
     */
    public function __construct(
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Framework\Registry $coreRegistry,
        \Magento\Backend\Model\Auth\Session $authSession,
        \Pinpoint\BlankProduct\Helper\Quantity $blankProductHelper
    ) {
        $this->_productFactory = $productFactory;
        $this->_coreRegistry = $coreRegistry;
        $this->_authSession = $authSession;
        $this->_blankProductHelper = $blankProductHelper;
    }

    /**
     * @param \Magento\Catalog\Model\Product $subject
     * @param $result
     * @return bool
     */
    public function afterIsSalable(
        \Magento\Catalog\Model\Product $subject,
        $result
    ) {
        if (!$this->_authSession->isLoggedIn()) {
            if ($this->_coreRegistry->registry(self::STOCKITEM_SALEABLE_REGISTRY) !== 'value') {
                $this->_coreRegistry->register(self::STOCKITEM_SALEABLE_REGISTRY, 'value');
                if ($result == false) {
                    $blankProduct = $this->_blankProductHelper->getBlankQuantityAvailable($subject->getId());
                    if ($blankProduct) {
                        $this->_coreRegistry->unregister(self::STOCKITEM_SALEABLE_REGISTRY);
                        return $blankProduct;
                    }
                }
                $this->_coreRegistry->unregister(self::STOCKITEM_SALEABLE_REGISTRY);
            }
        }

        return $result;
    }
}